from pydantic import BaseModel, ConfigDict, Field
from .category import Category
from .product import Product, ShortProduct


class ParentCategoriesResponse(BaseModel):
    categories: list[Category]


class CategoriesAndProductsBySlugParentCategory(BaseModel):
    date: str
    slug: str
    name: str
    url: str
    categories: list[Category]
    products: list[Product]


class NewProductsResponse(BaseModel):
    class Data(BaseModel):
        products: list[Product]

    class Meta(BaseModel):
        page: str
        totalPages: int
        totalCount: int
        pageSize: int

    data: Data
    _meta: Meta


class RemainsAndPlanresiduesResponse(BaseModel):
    date: str
    products: list[ShortProduct]


class TreeCategoriesResponse(BaseModel):
    tree: list[Category]



import requests

from bpiek.models.category import Category
from bpiek.models.product import Product
from bpiek.models.response import (
    CategoriesAndProductsBySlugParentCategory,
    NewProductsResponse,
    ParentCategoriesResponse,
    RemainsAndPlanresiduesResponse,
    TreeCategoriesResponse,
)

AUTH_URL = "https://bp.iek.ru/oauth/login"
API_URL = "https://bp.iek.ru/api/catalog/v1/"


class BPIekApi:
    def __init__(self) -> None:
        self.session = requests.Session()

    def _instance(self, endpoint: str, params: dict = {}):
        response = self.session.get(
            url=API_URL + endpoint,
            headers={"Content-Type": "application/json"},
            params={"format": "json", **params},
        )
        return response.json()

    def login(self, username: str, password: str) -> None:
        auth = self.session.post(
            url=f"{AUTH_URL}",
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            data={"username": username, "password": password},
        )

        if len(auth.cookies) == 0:
            raise Exception("Invalid username or password")

    def get_parent_categories(self) -> list[Category]:
        response = self._instance("client/catalog")

        try:
            result: ParentCategoriesResponse = ParentCategoriesResponse.model_validate(
                response
            )

            return result.categories

        except Exception as e:
            raise Exception(e)

    def get_product_by_article(self, article: str) -> Product:
        response = self._instance(f"client/products/{article}")

        try:
            result: Product = Product.model_validate(response)

            return result

        except Exception as e:
            raise Exception(e)

    def get_categories_and_products_by_slug_parent_category(
        self, slug: str
    ) -> CategoriesAndProductsBySlugParentCategory:
        response = self._instance(f"client/category/{slug}/json")

        try:
            result: CategoriesAndProductsBySlugParentCategory = (
                CategoriesAndProductsBySlugParentCategory.model_validate(response)
            )

            return result

        except Exception as e:
            raise Exception(e)

    def get_new_products(
        self,
        sortBy: str = "article",
        sortOrder: str = "asc",
        pageSize: int = 10,
        page: int = 1,
    ) -> NewProductsResponse:
        response = self._instance(
            "new-products",
            {sortBy: sortBy, sortOrder: sortOrder, pageSize: pageSize, page: page},
        )

        try:
            result: NewProductsResponse = NewProductsResponse.model_validate(response)

            return result

        except Exception as e:
            raise Exception(e)

    def get_remains_and_planresidues(self, slug: str) -> RemainsAndPlanresiduesResponse:
        response = self._instance(f"client/category/{slug}/balances-json")

        try:
            result: RemainsAndPlanresiduesResponse = (
                RemainsAndPlanresiduesResponse.model_validate(response)
            )

            return result

        except Exception as e:
            raise Exception(e)

    def get_tree_categories(self, parent_code: str = "00") -> list[Category]:
        tree = {}

        def convert_to_nested_list(nested_dict: dict, parent_code: str = "00") -> list:
            result = []
            for idx, (key, value) in enumerate(nested_dict.items()):
                if isinstance(value, dict):
                    try:
                        code = (
                            f"{parent_code}.{str(idx).zfill(2)}"
                            if parent_code
                            else f"{str(idx).zfill(2)}"
                        )
                        result.append(
                            {
                                "name": value["name"],
                                "slug": value["slug"],
                                "url": value["url"],
                                "code": code,
                                "childs": convert_to_nested_list(value["childs"], code),
                            }
                        )
                    except:
                        continue
                else:
                    result.append(value)
            return result

        def convert_list_to_nested_dict(_list: list):
            def update_nested(d, keys, value):
                for key in keys[:-1]:
                    d = d.setdefault(key, {})
                d[keys[-1]] = value

            tmp_categories = _list.copy()

            for idx, category in enumerate(tmp_categories):
                paths = category.url[1:][:-1].split("/")

                _str = ""
                for _idx, path in enumerate(paths):
                    if len(paths) == _idx + 1:
                        _str += f"{path}"
                        break

                    _str += f"{path},childs,"

                try:
                    update_nested(
                        tree,
                        _str.split(","),
                        {
                            "name": category.name,
                            "slug": category.slug,
                            "url": category.url,
                            "childs": {},
                        },
                    )

                    tmp_categories.pop(idx)
                except Exception as e:
                    print(e)
                    continue

            return tmp_categories

        for parent_category in self.get_parent_categories():
            tree[parent_category.slug] = {
                "name": parent_category.name,
                "slug": parent_category.slug,
                "url": parent_category.url,
                "childs": {},
            }
            categories_and_products = (
                self.get_categories_and_products_by_slug_parent_category(
                    slug=parent_category.slug
                )
            )
            tmp_categories = convert_list_to_nested_dict(
                categories_and_products.categories
            )
            while len(tmp_categories) != 0:
                tmp_categories = convert_list_to_nested_dict(tmp_categories)

        return TreeCategoriesResponse.model_validate(
            {"tree": convert_to_nested_list(tree, parent_code)}
        ).tree



from typing import Any

from pydantic import BaseModel, ConfigDict, Field
from pydantic.types import UUID1


class Category(BaseModel):
    code: str | None = Field(default=None)  # Код категории
    slug: str  # Слаг категории
    name: str  # Название категории
    url: str  # Относительный адрес категории
    apiUrl: str | None = Field(
        default=None
    )  # Ссылка на скачивание файла с содержимым категории
    childs: list["Category"] | None = Field(default=[])



from typing import Any

from pydantic import BaseModel, ConfigDict, Field


class WarehouseData(BaseModel):
    class Incoming(BaseModel):
        dateBegan: str | None
        dateEnd: str | None
        amount: int | float
        type: str  # Enum: "production" "shipping" Тип поступления, production - поступление после производства, shipping - доставка на склад

    warehouseId: str
    warehouseName: str
    availableAmount: int
    incoming: list[Incoming]


class ImageVariant(BaseModel):
    url: str  # Ссылка
    ext: str  # Расширение
    width: int  # Ширина


class Etim(BaseModel):
    class EtimClass(BaseModel):
        id: str
        name: str  # Название класса

    class EtimFeatures(BaseModel):
        id: str
        name: str  # Название свойства
        sort: int | None  # Порядок сортировки по умолчанию
        unit: str | None  # Единицы измерения
        value: str  # Значение свойства

    etim_class: EtimClass = Field(alias="class")
    features: list[EtimFeatures]


class Complects(BaseModel):
    article: str  # Артикул
    name: str  # Наименование
    quantity: int  # Количество


class LeftPeriod(BaseModel):
    name: str  # Название характеристики
    value: str | None  # Значение характеристики


class LeftPeriodRaw(BaseModel):
    class Lifespan(BaseModel):
        limit: str | None
        value: str | None
        units: str | None

    class Warranty(BaseModel):
        value: str | None
        units: str | None

    lifespan: Lifespan
    warranty: Warranty


class LogisticParams(BaseModel):
    class Value(BaseModel):
        group: str | None
        individual: str | None
        transport: str | None

    name: str
    nameOrig: str
    value: Value


class LogisticParamsData(BaseModel):
    class SinglePackage(BaseModel):
        multiplicity: int | None
        unit: str | None

    singlePackage: SinglePackage


class DesignFeatures(BaseModel):
    imageUrl: str
    description: str


class Videos(BaseModel):
    name: str
    description: str | None
    url: str
    type: str  # Enum: "url" "file"


class Software(BaseModel):
    name: str
    description: str
    url: str
    size: int


class AnalogProduct(BaseModel):
    article: str  # Артикул товара
    name: str  # Полное наименование товара
    shortName: str  # Краткое название
    description: str | None  # Описание
    imageUrl: str  # Фото товара (основное)
    imageUrls: list[str]  # Все фото товара
    imageVariants: list[ImageVariant]  # Все вариации изображений
    isArchived: bool  # Архивный или нет
    tm: str  # Торговая марка


class ShortProduct(BaseModel):
    article: str  # Артикул товара
    name: str  # Полное наименование товара
    multiplicity: int | None  # Кратность продажи
    priceBase: float | None  # Базовая цена с НДС
    priceRrc: float | None  # Рекомендованная розничная цена (РРЦ) с НДС
    priceRoc: float | None  # Розничная оптовая цена (РОЦ) с НДС
    available: int | None  # Значение остатка
    units: str | None  # Единицы измерения
    warehouseData: list[WarehouseData]


class Product(ShortProduct):
    shortName: str  # Краткое название
    description: str | None  # Описание
    categoryName: str | None  # Название категории
    category: str  # Относительный путь до категории в каталоге
    slug: str  # Слаг товара
    tm: str  # Торговая марка
    url: str  # Ссылка на товар
    isArchived: bool  # Архивный или нет
    imageUrl: str  # Фото товара (основное)
    imageUrls: list[str]  # Все фото товара
    imageVariants: list[ImageVariant]  # Все вариации изображений
    advantages: str | None  # Преимущества
    etim: Etim | None  # EIM характеристики товара
    complects: list[Complects]  # Комплектация и сопутствующие товары
    complectations: str | None  # Комплектация
    files: list[Any]  # Список файлов, относящихся к товару (ГЧ, КД, CAD-модели и т.д.)
    leftPeriod: list[LeftPeriod] | None  # Характеристики срока службы
    leftPeriodRaw: LeftPeriodRaw | None  # Гарантийные показатели
    logisticParams: list[LogisticParams]  # Логистические характеристики
    logisticParamsData: (
        LogisticParamsData | None
    )  # Подробные логистические характеристики
    novelty: bool  # Новинка или нет
    designFeatures: list[DesignFeatures]  # Отличительные особенности
    videos: list[Videos]  # Видео по товару
    software: list[Software]  # ПО по товару
    banner: str | None  # Текст баннера
    lastModified: str | None  # Дата последнего изменения
    countryOfProduction: str | None  # Страна производства
    firstSaleDate: str | None  # Дата начала продаж
    feacn: str | None  # Код ТН ВЭД
    family: str | Any | None = Field(default=None)
    series: str | Any | None = Field(default=None)
    indPacking: list[str]  # Ссылки на фото упаковки
    analogs: list[AnalogProduct]  # Аналоги
    related: list[AnalogProduct]  # Совместно применяемые изделия
    qrCode: str | None = Field(default=None)
    isOutOfAssortment: bool
    isOutOfProduction: bool
